#!/usr/bin/perl
#======================================================================
#    This source code is public domain, and may be freely distributed,
#    modified, and used for any purpose.  IT COMES WITH ABSOLUTELY NO
#    WARRANTY OF ANY KIND.
#======================================================================
# FILE:		make_skeleton.pl
# AUTHOR:	Craig A. James
# DESCRIPTION:
#	Example program illustrating the SMILES Toolkit.
#======================================================================

#!/usr/bin/perl

use strict;
use DayPerl2;

my $mol = dt_smilin($ARGV[0]);
make_skeleton($mol);
my $smiles = dt_cansmiles($mol, 1);
print "$smiles\n";
exit;

#----------------------------------------------------------------------
# SUBROUTINE: make_skeleton
#       Converts a molecule to a "skeleton" with all atoms replaced
#       by a C atom, and all bonds replaced by a single bond.
#----------------------------------------------------------------------

sub make_skeleton {
    my $molecule = shift;

    dt_mod_on($molecule);

    # Convert bonds to single
    my $bonds = stream2sequence($molecule, $TYP_BOND);
    while ($NULL_OB != (my $bond = dt_next($bonds))) {
        dt_setbondtype($bond, $DX_BTY_SINGLE);
    }

    # Convert atoms to wildcards
    my $atoms = stream2sequence($molecule, $TYP_ATOM);
    while ($NULL_OB != (my $atom = dt_next($atoms))) {
        dt_setnumber($atom, $DX_ATN_C);          # was DX_ATN_WILD, see above
        dt_setimp_hcount($atom, 0);
        dt_setcharge($atom, 0);
        dt_setaromatic($atom, 0);
        my $bcount = bond_count($atom);
        if ($bcount <= 4) {
            dt_setimp_hcount($atom, 4 - $bcount);
        }
    }
    dt_dealloc($atoms);

    # Done modifying the molecule
    if (!dt_mod_off($molecule)) {
        print STDERR "ERROR: make_skeleton(): dt_mod_off() failed\n";
    }
}

#-----------------------------------------------------------------
# METHOD: stream2sequence()
#    Creates a sequence object, copies the contents of a stream
#    to it.
#-----------------------------------------------------------------

sub stream2sequence {
    my ($molecule, $obtype) = @_;

    my $stream = dt_stream($molecule, $obtype);
    my $sequence = dt_alloc_seq();
    while ($NULL_OB != (my $ob = dt_next($stream))) {
        dt_append($sequence, $ob);
    }
    dt_reset($sequence);
    dt_dealloc($stream);
    return $sequence;
}

#----------------------------------------------------------------------
# SUBROUTINE: bond_count
#       Returns the sum of bond orders for an atom.
#----------------------------------------------------------------------

sub bond_count {
    my $atom = shift;
    my $bcount = 0;
    my $bonds = dt_stream($atom, $TYP_BOND);
    while ($NULL_OB != (my $bond = dt_next($bonds))) {
        $bcount += dt_bondorder($bond);
    }
    dt_dealloc($bonds);
    return $bcount;
}

