/* -*-C-*-
+======================================================================
| FILE:		test.c
| AUTHOR:	Craig A. James
| DESCRIPTION:
|	Test program for PNG drawing library, dl_ga.a
+======================================================================
*/

#include <stdio.h>
#include <string.h>
#include <dt_smiles.h>
#include <dt_depict.h>
#include "dl_gd.h"


main (int argc, char *argv[]) {

  dt_Handle mol, dep, atoms, atom, sob;
  char      *smi, *name;
  char      buf[2000];
  void      *image;
  int       imageSize, nbytes, totalBytes;


  /*** Initialize for PNG creation ***/
  dl_alloc_image(500, 400, 255, 255, 255, 1);
  dl_alloc_ga(1, 0, 200, 0);

  /*** Get SMILES, either from command line or stdin ***/

  if (2 != argc) {
    fgets(buf, 2000, stdin);
    buf[strlen(buf) - 1] = '\0';	/* zap newline */
    smi = buf;
  } else {
    smi = argv[1];
  }

  /*** Interpret argument as SMILES and make a depiction. ***/

  mol = dt_smilin(strlen(smi), smi);
  dep = dt_alloc_depiction(mol);
  dt_calcxy(dep);
  dt_setboolean(dep, 8, "suppress", 1);		/* Suppress C's of long-chain hydrocarbons */
  dt_setframe(dep, 0);				/* Suppress drawing box around molecule */

  /*** Set the atoms' color ***/
  atoms = dt_stream(mol, TYP_ATOM);
  while (NULL_OB != (atom = dt_next(atoms))) {
    dt_setga(dep, atom, 1);
  }
  dt_dealloc(atoms);
  dt_setga(dep, mol, 1);

  /*** Exit with warning on error. ***/
  if (NULL_OB == dep) {
    fprintf(stderr, "spewsmi: can't process `%s'\n", smi);
    exit(1);
  }

  /*** Add label if one specified. ***/
  if (NULL != (name = strchr(smi, ' '))) {
    name++;
    dt_setlabel1(dep, mol, name);
    dt_setlabel2(dep, mol, name);
    dt_setlabel1ga(dep, mol, 1);
    dt_setlabel2ga(dep, mol, 1);
  }

  /*** Create the image ***/
  dt_depict(dep);
  image = dl_get_image(&imageSize);
  if (imageSize <= 0) {
    fprintf(stderr, "Failed to get PNG file\n");
  } else {
    totalBytes = nbytes = 0;
    while (totalBytes < imageSize && nbytes >= 0) {
      nbytes = write(1, image, imageSize);
      totalBytes += nbytes;
    }
    fprintf(stderr, "Wrote %d byte PNG file.\n", totalBytes);
  }

  /*** Clean up. ***/
  dl_free_image();

  exit(0);
}
