// Hey, emacs, treat this as a -*- c++ -*- file!
//==========================================================================+
//    This source code is public domain, and may be freely distributed,     |
//    modified, and used for any purpose.  IT COMES WITH ABSOLUTELY NO      |
//    WARRANTY OF ANY KIND.						    |
//==========================================================================+
// 
// FILE:	typemaps.i
// AUTHOR:	Craig A. James
//		http://www.moonviewscientific.com/
// DESCRIPTION:
//     These typemaps modify the behavior of Swig to handle various "idioms"
//     in the C functions of the Daylight Toolkit.
//
//	Note: It is critical that you DO NOT include the standard typemaps.i
//	that comes with Swig for Perl.  It's definitions aren't suitable
//	for the Daylight Toolkit.
//===========================================================================


//----------------------------------------------------------------------
// STRINGS RETURNED FROM FUNCTIONS
//
// Daylight always returns strings as unterminated char * (i.e. no NULL
// terminator), and also returns their length as the first parameter, like
// this:
//
//     char *s = dt_foo(dt_Integer *slen, dt_Handle ob, ...)
//
// Fortunately, all Daylight functions that have this signature return
// a string.
//
//   - The first rule, "arginit", is invoked first by Swig and gives us a
//     chance to initialize variables -- in this case, to put an integer on
//     the stack to hold the returned string's length.
//  
//   - The second one, "argout", is put in the function after the actual
//     function call, and lets us copy the returned string's contents to a
//     new Perl variable.
//  
//   - The third typemap tells Swig that the plen parameters isn't really an
//     input at all, so the resulting Perl method will take fewer
//     argument than the C method.
// 
// The various "signatures" below catch all of the actual forms used in 
// the Daylight Toolkit .h files.
//----------------------------------------------------------------------

%typemap(perl5, arginit) (dt_Integer *lenp, dt_Handle ob) (int my_strlen),
			 (dt_Integer *plen, dt_Handle ob) (int my_strlen),
			 (dt_Integer *slen, dt_Handle ob) (int my_strlen),
  			 (dt_Integer *plen, dt_Handle sh) (int my_strlen),
  			 (dt_Integer *plen, dt_Handle colh) (int my_strlen),
  			 (dt_Integer *plen, dt_Handle hith) (int my_strlen),
  			 (dt_Integer *plen, dt_Handle colh) (int my_strlen),
			 (dt_Integer *plen, dt_Handle hh)   (int my_strlen),
			 (int        *dl_plen)              (int my_strlen)

%{
  $1 = &my_strlen;
%}

%typemap(perl5, argout)  (dt_Integer *lenp, dt_Handle ob),
			 (dt_Integer *plen, dt_Handle ob),
			 (dt_Integer *slen, dt_Handle ob),
			 (dt_Integer *plen, dt_Handle sh),
			 (int        *dl_plen)
{
  ST(argvi) = sv_newmortal();
  sv_setpvn((SV*)ST(argvi++), result, my_strlen$argnum);
}
%typemap(perl5, in, numinputs=0) (dt_Integer *lenp),
				 (dt_Integer *plen),
				 (int        *dl_plen),
				 (dt_Integer *slen)
%{ %}

//----------------------------------------------------------------------
// Tell Swig about the various "dt_Integer *isnew" parameters -- integers
// returned by some Daylight functions.  Using these typemaps, Perl
// functions become multi-valued returns.  For example, a C function 
// of the form:
//
//	dt_Handle dt_foo(dt_Handle h, dt_Integer *isnew);
//
// will convert to a Perl function that returns an array when used in an
// array context, for example:
//
//	($foo, $isnew) = dt_foo($h);
//
// The typemaps are:
//
//   * The first typemap creates a local variable on the stack for the
//     function to use, and points the pointer to it.  The returned
//     value will be stored in this stack variable.
//
//   * The second typemap creates a new Perl object and puts the 
//     returned value in it.
//
//   * The third typemap tells Swig that the "isnew" parameter isn't
//     really an input parameter, so the Perl function will have one
//     fewer parameters than the C function.
//
// The various "signatures" below catch all of the actual forms used
// in the Daylight Toolkit.
//----------------------------------------------------------------------

%typemap(perl5, arginit)
  dt_Integer *errcode		(int my_isnew),
  dt_Integer *isnew		(int my_isnew),
  dt_Integer *perror		(int my_isnew),
  dt_Integer *pnew		(int my_isnew),
  dt_Integer *pret_status	(int my_isnew),
  dt_Integer *pstatus		(int my_isnew),
  dt_Integer *ret_status	(int my_isnew),
  dt_Integer *return_status	(int my_isnew)
%{
  $1 = &my_isnew;
%}

%typemap(perl5, argout)
  dt_Integer *errcode,
  dt_Integer *isnew,
  dt_Integer *perror,
  dt_Integer *pnew,
  dt_Integer *pret_status,
  dt_Integer *pstatus,
  dt_Integer *ret_status,
  dt_Integer *return_status
%{
  ST(argvi) = sv_newmortal();
  sv_setiv((SV*)ST(argvi++), (IV) *$1);
%}

%typemap(perl5, in, numinputs=0)
  dt_Integer *errcode,
  dt_Integer *isnew,
  dt_Integer *perror,
  dt_Integer *pnew,
  dt_Integer *pret_status,
  dt_Integer *pstatus,
  dt_Integer *ret_status,
  dt_Integer *return_status
%{ %}

//----------------------------------------------------------------------
// Tell Swig about returned floating-point coordinates.  (Note: this
// is very similar to the *isnew rules, above).  There's
// only one of these: dt_getcoord().  It turns into a multi-valued
// return:
//
//	($x, $y, $z) = dt_getcoord($mol, $depiction);
//
//
// The typemaps are:
//
//   * The first typemap creates three local variables on the stack for
//     the function to use, and points the pointer to it.  The returned
//     values will be stored in these stack variables.
//
//   * The second typemap creates three new Perl objects and puts the 
//     returned value in it.
//
//   * The third typemap tells Swig that the "*px, *py, *pz" parameters
//     aren't really input parameters, so the Perl function will have 
//     three fewer parameters than the C function.
//----------------------------------------------------------------------

%typemap(perl5, arginit)
  (dt_Real *xp, dt_Real *yp, dt_Real *zp)	(dt_Real my_x, dt_Real my_y, dt_Real my_z)
%{
  $1 = &my_x;
  $2 = &my_y;
  $3 = &my_z;
%}

%typemap(perl5, argout)
  (dt_Real *xp, dt_Real *yp, dt_Real *zp)
%{
  ST(argvi) = sv_newmortal();
  sv_setnv(ST(argvi++), (double) *$1);
  ST(argvi) = sv_newmortal();
  sv_setnv(ST(argvi++), (double) *$2);
  ST(argvi) = sv_newmortal();
  sv_setnv(ST(argvi++), (double) *$3);
%}

%typemap(perl5, in, numinputs=0)
  (dt_Real *xp, dt_Real *yp, dt_Real *zp)
%{ %}


//----------------------------------------------------------------------
// Tell Swig about input strings.  The Daylight Toolkit always uses
// a string length with every input string, and the strings don't
// have to be null-terminated.  So we have to convert Perl's string to
// a string-plus-length.
//
//  * The first rule, "check", is somewhat of a hack.  The "check" rule is
//    supposed to be used to check parameters at run time, but it happens
//    to be the only Swig rule that runs after parameter conversion and
//    before the actual C function call.  So we use it to get the string's
//    length and assign it to the right argument.
//
//  * The second rule tells Swig that the "length" parameter isn't really
//    an input parameter, so that the Perl function only has a single
//    string parameter.
//
// By good fortune (and probably by design), it turns out that every
// dt_Integer parameter with 'len' as part of its name is the length
// of a string, so we can be sure the second rule won't accidentally
// get applied to some dt_Integer that isn't part of a length/string 
// pair.  There are three additional parameters, "passl", "pathl" and
// "perml" (used in dt_open()) that also are covered here.
//----------------------------------------------------------------------

%typemap(perl5, check)
	(dt_Integer alen,	dt_String ascii),
	(dt_Integer blen,       dt_String bin),
	(dt_Integer chortlen,   dt_String chortles),	
	(dt_Integer dirlen,     dt_String dir),
	(dt_Integer hlen,   	dt_String host),
	(dt_Integer idlen,	dt_String id),
	(dt_Integer len,   	dt_String name),
	(dt_Integer len,    	dt_String chart),
	(dt_Integer len,    	dt_String chort),
	(dt_Integer len,    	dt_String chuck),
	(dt_Integer len,        dt_String s),
	(dt_Integer len,        dt_String tag),
	(dt_Integer len,        dt_String tdtstr),
	(dt_Integer len,        dt_String value),
	(dt_Integer namlen, 	dt_String name),
	(dt_Integer nlen,       dt_String name),
	(dt_Integer pathlen,    dt_String path),
	(dt_Integer pathl,      dt_String path),
	(dt_Integer passl,      dt_String pass),
	(dt_Integer perml,      dt_String perm),
	(dt_Integer plen,   	dt_String passwd),
	(dt_Integer plen,   	dt_String password),
	(dt_Integer prlen,      dt_String product),
	(dt_Integer pwlen,      dt_String pw),
	(dt_Integer pwlen,      dt_String password),
	(dt_Integer s1len,  	dt_String s1),
	(dt_Integer s2len,  	dt_String s2),
	(dt_Integer slen,   	dt_String service),
	(dt_Integer slen,   	dt_String str),
	(dt_Integer slen,       dt_String string),
	(dt_Integer strlen,     dt_String string),
	(dt_Integer strlen,     dt_String str),
	(dt_Integer smilen, 	dt_String smiles),
	(dt_Integer slen, 	dt_String smiles),
	(dt_Integer symlen, 	dt_String symbol),
	(dt_Integer symlen, 	dt_String symbol),
	(dt_Integer symlen,     dt_String symbol),
	(dt_Integer taglen,   	dt_String tag),
	(dt_Integer ulen,   	dt_String userid),
	(dt_Integer ulen,   	dt_String user),
	(dt_Integer ulen,   	dt_String username),
	(dt_Integer vlen,   	dt_String value)
%{
        if (0 == $2) {
	  $1 = 0;
        } else {
	  $1 = strlen($2);
	}
%}

%typemap(perl5, in, numinputs=0)
	(dt_Integer alen),
	(dt_Integer blen),
	(dt_Integer chortlen),	
	(dt_Integer dirlen),
	(dt_Integer hlen),
	(dt_Integer idlen),
	(dt_Integer len),
	(dt_Integer namlen),
	(dt_Integer nlen),
	(dt_Integer pathlen),
	(dt_Integer pathl),
	(dt_Integer passl),
	(dt_Integer perml),
	(dt_Integer plen),
	(dt_Integer prlen),
	(dt_Integer pwlen),
	(dt_Integer s1len),
	(dt_Integer s2len),
	(dt_Integer slen),
	(dt_Integer strlen),
	(dt_Integer smilen),
	(dt_Integer symlen),
	(dt_Integer taglen),
	(dt_Integer ulen),
	(dt_Integer vlen)
%{
%}
