#!/bin/sh
#==========================================================================+
#    This source code is public domain, and may be freely distributed,     |
#    modified, and used for any purpose.  IT COMES WITH ABSOLUTELY NO      |
#    WARRANTY OF ANY KIND.						   |
#==========================================================================+
#
# FILE:		Build.sh
# AUTHOR:	Craig A. James
# DESCRIPTION:
#	Compiles the libraries needed for DayPerl2's drawing library.
#======================================================================

set -e

function say_title() {
    echo ""
    echo "--------------------------------------------------"
    echo $1
    echo "--------------------------------------------------"
    echo ""
}


DIRS="include lib bin man share"
echo "Creating directories: $DIRS ..."
for dir in $DIRS ; do
    if [ ! -d $dir ] ; then
	echo "mkdir $dir"
	mkdir $dir
    fi
done

# Gnu make(1) is required to compile libfreetype.  If the default make(1)
# isn't Gnu make, then compile it here.

make_version=`make -v | head -1 | cut -d' ' -f1`
if [ "$make_version" != "GNU" ] ; then
    local_gnumake=1
    MAKE=$INSTALL_DIR/bin/make
else
    MAKE=make
fi
export MAKE

#----------------------------------------------------------------------
# Unpack all the archives
#----------------------------------------------------------------------

say_title "Unpacking Archives"

if [ ! -z "$local_gnumake" ] ; then
    echo "Unpacking make..."
    gunzip < make-3.80.tar.gz | tar xf -
    mv make-3.80 gnumake
fi

echo "Unpacking libfreetype..."
gunzip <freetype-2.1.5.tar.gz | tar xf -
mv freetype-2.1.5 freetype

echo "Unpacking libgd..."
gunzip <gd-2.0.28.tar.gz | tar xf -
mv gd-2.0.28 libgd

echo "Unpacking zlib..."
gunzip <zlib-1.1.4.tar.gz | tar xf -
mv zlib-1.1.4 zlib

echo "Unpacking JPG library..."
gunzip <jpegsrc.v6b.tar.gz | tar xf -

echo "Unpacking libpng..."
gunzip <libpng-1.2.5.tar.gz | tar xf -
mv libpng-1.2.5 libpng

if [ ! -z "$local_gnumake" ] ; then
    say_title "Compiling GNU make"
    (
	cd gnumake
	INSTALL_DIR=`cd ..; pwd`
	./configure --prefix=$INSTALL_DIR
	make
	make install
    )
fi

#----------------------------------------------------------------------
# Compile everything.  Some of these must come before others.
#----------------------------------------------------------------------

say_title "Compiling libfreetype"
(
  cd freetype
  INSTALL_DIR=`cd ..; pwd`
  ./configure --prefix=$INSTALL_DIR
  $MAKE
  $MAKE install
)

say_title "Compiling zlib"
(
  cd zlib
  ./configure
  $MAKE
  $MAKE install prefix=..
)

say_title "Compiling libpng"
(
  ostype=`uname -s | tr '[A-Z]' '[a-z]'`
  cd libpng
  cp scripts/makefile.${ostype} ./makefile
  $MAKE
  $MAKE install prefix=..
)

say_title "Compiling JPEG library"
(
  cd jpeg-6b
  ./configure
  $MAKE
  cp libjpeg.a ../lib
)

say_title "Compiling libgd"
(
  cd libgd
  INSTALL_DIR=`cd ..; pwd`
  ./configure --with-png=.. --with-jpeg=.. --with-gif=. --with-freetype=.. --prefix=$INSTALL_DIR
  $MAKE
  $MAKE install
)

say_title "Done."
